/* START OF user_config.c ------------------------------------------------------
 *
 * 	Locate/create user configuration files.
 *
 * 	There are two versions of the below, a UNIX version (WITH_UNIX_EXTRAS)
 *	and an ANSI C version.  The ANSI C version will fail (#error) where you
 *	need to add OS specific code.
 *
 *	(It'll only support UNIX and <other>, not multiple systems, rethink it.)
 *
 *	(This again is not fantastic code, but if I'm going to get anywhere with
 *	this I'm going to have to speedup development a bit.)
 *
 *      ---
 *      Written by David Eggleston (2006) <deggleston@users.sourceforge.net>
 *      for the 'BeebEm' Acorn BBC Model B, Integra-B, Model B Plus and
 *      Master 128 emulator.
 *
 *      This file is part of BeebEm and may be copied only under the terms of
 *	either the GNU General Public License (GPL) or Dr. David Alan Gilbert's
 *	BeebEm license.
 *
 *      For more details please visit:
 *
 *      http://www.gnu.org/copyleft/gpl.html
 *      ---
 *
 *	Shouldn't use BOOL here (it's defined in SDL).
 */

#if HAVE_CONFIG_H
#       include <config.h>
#endif

#include "include/functions.h"
#include "include/user_config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Extra UNIX specific includes:
 */
#ifdef WITH_UNIX_EXTRAS
#	include <sys/types.h>
#	include <unistd.h>
#	include <pwd.h>
#	include <errno.h>
#	include <sys/stat.h>
#	include <dirent.h>
#endif



/* Sub directory (from DATA_DIR) for user configuration directory when not UNIX.
 */
#define NONE_UNIX_CONFIG_DIR "config"


/* Max length a path (including filename) can be for target OS.
 * 
 * [TODO] Move this to configure.in so can be modified at build time.
 */
#define MAX_PATH_LEN	(1024*4+1)

/* Filename and path manipulation functions, not too formal, but does handle
 * more ovious errors.
 */
static BOOL AppendStringToPath(char *path, char *append, size_t len)
{
	/* Don't bother checking if NULL, but do warn.
	 */
	if (path == NULL){
		qWARN("Appending to NULL path.");
		return(FALSE);
	}
	if (append == NULL){
		qWARN("Appending NULL to path.");
		return(FALSE);
	}

	/* Will the characters to append fit?
	 */
	if (strlen(path)+strlen(append) < len)
		/* Yes, so append it.
		 */
		strcat(path, append);
	else{
		qERROR("Path too long.");
		return(FALSE);
	}

	return(TRUE);
}

/* Will append a forward slash to the path if one does not already exist
 */
static BOOL AppendForwardSlashToPath(char *path, size_t len)
{
	/* Don't bother if path NULL.
	 */
	if (path == NULL){
		qWARN("Appending directory to NULL path.");
		return(FALSE);
	}

	if (strlen(path)>0 && path[strlen(path)-1] != '/')
		return( AppendStringToPath(path, "/", len) );

	return(TRUE);
}

/* Will append a directory name to a path (and handle forward slash).  The
 * directory name should not contain slashes.
 */
static BOOL AppendDirToPath(char *path, char *dir, size_t len)
{
	/* Don't bother if NULL.
	 */
	if (path == NULL){
		qWARN("Appending directory to NULL path.");
		return(FALSE);
	}
	if (dir == NULL){
		printf("Appending NULL directory to path.");
		return(FALSE);
	}

	/* Append "/" to path if needed.
	 */
	if (! AppendForwardSlashToPath(path, len) )
		return(FALSE);

	/* Append Directory to Path/
	 */
	if (! AppendStringToPath(path, dir, len) )
		return(FALSE);

	/* As it's a dir, make sure a forward slash is appended too.
	 */
	if (! AppendForwardSlashToPath(path, len) )
		return(FALSE);

	return(TRUE);
}

/* Will append a filename to a path.
 */
static BOOL AppendFilenameToPath(char *path, char *filename, size_t len)
{
        /* Don't bother if NULL.
         */
        if (path == NULL){
                qWARN("Appending directory to NULL path.");
                return(FALSE);
        }
        if (filename == NULL){
                qWARN("Appending NULL filename to path.");
                return(FALSE);
        }

	/* Append "/" to path if needed.
	 */
	if (! AppendForwardSlashToPath(path, len) )
		return(FALSE);

	/* Append filename to Path
	 */
	if (! AppendStringToPath(path, filename, len) )
		return(FALSE);

	return(TRUE);
}

/*	*	*	*	*	*	*	*	*	*
 */


/* Test whether a regualr file exists.
 */
static BOOL TestFile(char *file)
{
	FILE *f;

	if ( (f=fopen(file, "r")) == NULL)
		return(FALSE);

	fclose(f);
	return(TRUE);
}


/* Test whether a directory entry exists.
 */
static BOOL TestDirectory(char *file)
{
#ifdef WITH_UNIX_EXTRAS
	DIR *dir;

	if ( (dir=opendir(file)) == NULL)
		return(FALSE);
	
	closedir(dir);
	return(TRUE);
#else
#	error You need to add TestDirectory support for your OS here.
#endif
}


/* Make a copy of a file.
 *
 * Both paths must be fully quantified (leading "/") for security reasons.
 */
static BOOL CopyFile(char *source_file, char *dest_file)
{
	/* Terminate program if file paths are not fully quantified.
	 *
	 * (They always should be for BeebEm, and if they're not, then it could
	 * be a security risk).
	 */
	if ( source_file == NULL || dest_file == NULL
	 || strlen(source_file) == 0 || strlen(dest_file) == 0
	 || source_file[0] != '/' || dest_file[0] != '/') {
		qFATAL("CopyFile arguments are not fully quantified"
		 " paths (bailing out).");
		exit(1);
	}

#ifdef WITH_UNIX_EXTRAS
	char command[5*1024];

	sprintf(command, "cp %s %s", source_file, dest_file);
	if (system(command) != 0)
		return(FALSE);
#else
#	error You need to add CopyFile support for your OS here.
#endif
	return(TRUE);
}


/* Create a new directory (succeeds if dir already exists).
 */
static BOOL CreateDirectory(char *file)
{
#ifdef WITH_UNIX_EXTRAS
	if (TestDirectory(file))
		return(TRUE);

	errno = 0;
	if (mkdir(file, (mode_t) S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP
	 | S_IXGRP | S_IROTH | S_IXOTH ) == 0)
		return(TRUE);

	pFATAL(dL"Could not create directory: %s", dR, strerror(errno));
	return(FALSE);
#else
#	error You need to add CreateDirectory support for your OS here.
#endif
}

/* Get users config directory ("${DATA_DIR}/config/" or "$HOME/.beebem/),
 * does add a trailing forward slash.
 *
 * On UNIX for a user called dave, would return a pointer to something like:
 * "/home/dave/.beebem/".  If this directory does not exist, it's created.
 *
 * On None UNIX systems, the installation path of the application + a config
 * directory name are used instead.  So something like:
 * "/applications/beebem/config/" will be returned instead.
 */
char* GetUserConfigPath(char *path, size_t len)
{
#ifdef WITH_UNIX_EXTRAS
	struct passwd *pw_ptr;

	/* Find user in password database.
	 */
	errno = 0; 
	if ( ! (pw_ptr=getpwuid(getuid())) ){
		/* Not found, clear config filename.
		 */
		pFATAL(dL"Unable to determine user home dir: %s", dR, strerror(errno));
		exit(1);
	}else{

		if (! AppendDirToPath(path, pw_ptr->pw_dir, len) ){
			qFATAL("Unable to accept user accounts dir.");
			exit(1);
		}

		if (! AppendDirToPath(path, "." PACKAGE, len) ){
			qFATAL("Unable to append ." PACKAGE " to user accounts dir.");
			exit(1);
		}
	}
#else
	/* Root of application data dir
	 */
	
	if (! AppendDirToPath(path, DATA_DIR, len) ){
		qFATAL("Unable to determine default user config dir.");
		exit(1);
	}

	/* Add config dir
	 */
	if (! AppendDirToPath(path,NONE_UNIX_CONFIG_DIR, len) ){
		qFATAL("Unable to accept default user config dir.");
		exit(1);
	}
#endif

	/* If configuration directory does not exist, create it.
	 */
	if (! TestDirectory(path) )
		if (! CreateDirectory(path)){
			qFATAL("Unable to create user config dir.");
			exit(1);
		}

	return(path);
}


/* The following functions are called within the emulator core to determine
 * the location of config files.
 *
 * If the file does not exist, most of these functions will try to create it
 * first.
 */


// Get (+create if does not exist) roms.cfg filename and path.
// /config/roms.cfg		if ~/.beebem/roms.cfg does not exist create it.
#define ROM_CONFIG_FILE "roms.cfg"	// Put these in configure.in?
char* GetLocation_roms_cfg(char *buffer, size_t length)
{
	// On error return NULL, otherwise &buffer.

	// Get user config dir root.
	// Test for ROM_CONFIG_FILE.
	// If not there, copy from shared.
	// Write path and filename to buffer
}


// Get (+create if does not exist) econet.cfg filename and path.
// /config/econet.cfg		if ~/.beebem/econet.cfg does not exist create it
char* GetLocation_econet_cfg(char *buffer, size_t length)
{
}


// Get (+create if does not exist) phroms.cfg filename and path.
// /config/phroms.cfg		if ~/.beebem/phroms.cfg does not exist create it
char* GetLocation_phroms_cfg(char *buffer, size_t length)
{
}


// Get (but do not create if does not exist) cmos.ram filename and path.
// /state/cmos.ram		TBD.
char* GetLocation_cmos_ram(char *buffer, size_t length)
{
}


// Get /roms/ path.  If ~/.beebem/roms exists use that, otherwise use the global
// one.
// /roms/*			if ~/.beebem/roms exists use this one instead.
char* GetLocation_roms(char *buffer, size_t length)
{
}


// Get /media/scsi/
// /media/scsi/*		if ~/.beebem/media/scsi/* does not exist, 
// create it.
char* GetLocation_scsi(char *buffer, size_t length)
{
}


